@@ -336,12 +336,13 @@ gdip_linear_gradient_setup (GpGraphics *graphics, GpBrush *brush)
 	if (linear->base.changed || !linear->pattern) {
 
 		/* destroy the existing pattern */
-		if (linear->pattern)
+		if (linear->pattern) {
 			cairo_pattern_destroy (linear->pattern);
-
-		if (linear->wrapMode == WrapModeClamp) {
-			return InvalidParameter;
+			linear->pattern = NULL;
 		}
+
+		if (linear->wrapMode == WrapModeClamp)
+			return InvalidParameter;
 			
 		status = create_tile_linear (graphics, ct, linear);
 
@@ -520,7 +521,7 @@ GdipCreateLineBrush (GDIPCONST GpPointF *point1, GDIPCONST GpPointF *point2, ARG
 
 	linear->points [0].X = linear->rectangle.X;
 	linear->points [0].Y = linear->rectangle.Y;
-	linear->points [1].X = linear->rectangle.X + linear->rectangle.Width + 1;
+	linear->points [1].X = linear->rectangle.X + linear->rectangle.Width;
 	linear->points [1].Y = linear->rectangle.Y;
 
 	linear->angle = linear->angle * DEGTORAD;
