@@ -20,8 +20,11 @@
  *	Sebastien Pouliot  <sebastien@ximian.com>
  */
 
-#include "metafile.h"
-
+#include "metafile-private.h"
+#include "solidbrush-private.h"
+#include "graphics.h"
+#include "graphics-path-private.h"
+#include "pen.h"
 
 //#define DEBUG_METAFILE
 
@@ -594,7 +597,7 @@ gdip_metafile_StretchDIBits (MetafilePlayContext *context, int XDest, int YDest,
 	printf ("\n\t\tClrUsed: %d", lpBitsInfo->bmiHeader.biClrUsed);
 	printf ("\n\t\tClrImportant: %d", lpBitsInfo->bmiHeader.biClrImportant);
 #endif
-	ms.ptr = (unsigned char*)lpBitsInfo;
+	ms.ptr = (BYTE*)lpBitsInfo;
 	ms.size = lpBitsInfo->bmiHeader.biSizeImage;
 	ms.pos = 0;
 	status = gdip_read_bmp_image (&ms, &image, Memory);
@@ -851,10 +854,13 @@ gdip_metafile_create ()
 {
 	GpMetafile* mf = (GpMetafile*) GdipAlloc (sizeof (GpMetafile));
 	if (mf) {
-		mf->base.type = imageMetafile;
+		mf->base.type = ImageTypeMetafile;
 		mf->delete = FALSE;
 		mf->data = NULL;
 		mf->length = 0;
+		mf->recording = FALSE;
+		mf->fp = NULL;
+		mf->stream = NULL;
 	}
 	return mf;
 }
@@ -890,10 +896,31 @@ gdip_metafile_dispose (GpMetafile *metafile)
 		metafile->data = NULL;
 	}
 
+	if (metafile->recording)
+		gdip_metafile_stop_recording (metafile);
+
 	GdipFree (metafile);
 	return Ok;
 }
 
+GpStatus
+gdip_metafile_stop_recording (GpMetafile *metafile)
+{
+	/* TODO save current stuff */
+
+	if (metafile->fp) {
+		fclose (metafile->fp);
+		metafile->fp = NULL;
+	}
+	if (metafile->stream) {
+		/* it's not ours to close, just forget about it */
+		metafile->stream = NULL;
+	}
+	/* we cannot open a new graphics instance on this metafile - recording is over */
+	metafile->recording = FALSE;
+	return Ok;
+}
+
 MetafilePlayContext*
 gdip_metafile_play_setup (GpMetafile *metafile, GpGraphics *graphics, int x, int y, int width, int height)
 {
@@ -988,7 +1015,7 @@ gdip_metafile_play (MetafilePlayContext *context)
 {
 	GpStatus status;
 
-	if (!context)
+	if (!context || !context->metafile)
 		return InvalidParameter;
 
 	switch (context->metafile->metafile_header.Type) {
@@ -1612,6 +1639,166 @@ GdipSetMetafileDownLevelRasterizationLimit (GpMetafile *metafile, UINT metafileR
 GpStatus
 GdipPlayMetafileRecord (GDIPCONST GpMetafile *metafile, EmfPlusRecordType recordType, UINT flags, UINT dataSize, GDIPCONST BYTE* data)
 {
+	if (!metafile)
+		return InvalidParameter;
+
 	/* TODO */
 	return NotImplemented;
 }
+
+GpStatus
+GdipRecordMetafile (HDC referenceHdc, EmfType type, GDIPCONST GpRectF *frameRect, MetafileFrameUnit frameUnit, 
+	GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	GpMetafile *mf;
+
+	if (!referenceHdc || !frameRect || !metafile)
+		return InvalidParameter;
+
+	if ((type < EmfTypeEmfOnly) || (type > EmfTypeEmfPlusDual))
+		return InvalidParameter;
+
+	if ((frameUnit < MetafileFrameUnitPixel) || (frameUnit > MetafileFrameUnitGdi))
+		return InvalidParameter;
+
+	if (((frameRect->Width == 0) || (frameRect->Height == 0)) && (frameUnit != MetafileFrameUnitGdi))
+		return GenericError;
+
+	mf = gdip_metafile_create ();
+	if (!mf)
+		return OutOfMemory;
+
+	mf->metafile_header.X = frameRect->X;
+	mf->metafile_header.Y = frameRect->Y;
+	mf->metafile_header.Width = frameRect->Width;
+	mf->metafile_header.Height = frameRect->Height;
+	mf->metafile_header.Size = 0;
+	mf->metafile_header.Type = type;
+	mf->recording = TRUE;
+
+	/* TODO - more stuff here! */
+
+	*metafile = mf;
+	return Ok;
+}
+
+GpStatus
+GdipRecordMetafileI (HDC referenceHdc, EmfType type, GDIPCONST GpRect *frameRect, MetafileFrameUnit frameUnit, 
+	GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	GpRectF rect;
+
+	if (!frameRect)
+		return InvalidParameter;
+
+	rect.X = frameRect->X;
+	rect.Y = frameRect->Y;
+	rect.Width = frameRect->Width;
+	rect.Height = frameRect->Height;
+	return GdipRecordMetafile (referenceHdc, type, (GDIPCONST GpRectF*) &rect, frameUnit, description, metafile);
+}
+
+GpStatus
+GdipRecordMetafileFileName (GDIPCONST WCHAR *fileName, HDC referenceHdc, EmfType type, GDIPCONST GpRectF *frameRect, 
+	MetafileFrameUnit frameUnit, GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	GpStatus status;
+	GpMetafile *mf = NULL;
+	char *file_name;
+
+	if (!fileName)
+		return InvalidParameter;
+
+	file_name = (char *) ucs2_to_utf8 ((const gunichar2 *)fileName, -1);
+	if (!file_name) {
+		*metafile = NULL;
+		return InvalidParameter;
+	}
+
+	status = GdipRecordMetafile (referenceHdc, type, frameRect, frameUnit, description, &mf);
+	if (status != Ok) {
+		GdipFree (file_name);
+		return status;
+	}
+
+	/* yep, an existing file is overwritten */
+	mf->fp = fopen (file_name, "wb");
+
+	GdipFree (file_name);
+	*metafile = mf;
+	return Ok;
+}
+
+GpStatus
+GdipRecordMetafileFileNameI (GDIPCONST WCHAR *fileName, HDC referenceHdc, EmfType type, GDIPCONST GpRect *frameRect, 
+	MetafileFrameUnit frameUnit, GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	GpRectF rect;
+
+	if (!frameRect)
+		return InvalidParameter;
+
+	rect.X = frameRect->X;
+	rect.Y = frameRect->Y;
+	rect.Width = frameRect->Width;
+	rect.Height = frameRect->Height;
+	return GdipRecordMetafileFileName (fileName, referenceHdc, type, (GDIPCONST GpRectF*) &rect, frameUnit, description, metafile);
+}
+
+/*
+ * GdipRecordMetafileStream and GdipRecordMetafileStreamI will never be implemented, as 'stream' is a COM IStream ...
+ */
+GpStatus
+GdipRecordMetafileStream (void /* IStream */ *stream, HDC referenceHdc, EmfType type, GDIPCONST GpRectF *frameRect, 
+	MetafileFrameUnit frameUnit, GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	return NotImplemented;
+}
+
+GpStatus
+GdipRecordMetafileStreamI (void /* IStream */ *stream, HDC referenceHdc, EmfType type, GDIPCONST GpRect *frameRect, 
+	MetafileFrameUnit frameUnit, GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	return NotImplemented;
+}
+/*
+ * instead we'll use delegates to create the metafile header with these functions
+ */
+GpStatus 
+GdipRecordMetafileFromDelegate_linux (GetHeaderDelegate getHeaderFunc, GetBytesDelegate getBytesFunc,
+	PutBytesDelegate putBytesFunc, SeekDelegate seekFunc, CloseDelegate closeFunc, SizeDelegate sizeFunc, 
+	HDC referenceHdc, EmfType type, GDIPCONST GpRectF *frameRect, 
+	MetafileFrameUnit frameUnit, GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	GpStatus status;
+
+	if (!putBytesFunc)
+		return InvalidParameter;
+
+	status = GdipRecordMetafile (referenceHdc, type, frameRect, frameUnit, description, metafile);
+	if (status != Ok)
+		return status;
+
+	/* TODO - keep delegates around to write stuff */
+
+	return Ok;
+}
+
+GpStatus 
+GdipRecordMetafileFromDelegateI_linux (GetHeaderDelegate getHeaderFunc, GetBytesDelegate getBytesFunc,
+	PutBytesDelegate putBytesFunc, SeekDelegate seekFunc, CloseDelegate closeFunc, SizeDelegate sizeFunc, 
+	HDC referenceHdc, EmfType type, GDIPCONST GpRect *frameRect, 
+	MetafileFrameUnit frameUnit, GDIPCONST WCHAR *description, GpMetafile **metafile)
+{
+	GpRectF rect;
+
+	if (!frameRect)
+		return InvalidParameter;
+
+	rect.X = frameRect->X;
+	rect.Y = frameRect->Y;
+	rect.Width = frameRect->Width;
+	rect.Height = frameRect->Height;
+	return GdipRecordMetafileFromDelegate_linux (getHeaderFunc, getBytesFunc, putBytesFunc, seekFunc, closeFunc, sizeFunc,
+		referenceHdc, type, (GDIPCONST GpRectF*) &rect, frameUnit, description, metafile);
+}
